###
### Rules.make -- General Rules for making Tunes
###


.PHONY:		phony all config autoconf dep depend \
		Tunes test install clean veryclean mrproper tgz tar

.SUFFIXES:	.hlm .scm .v .scm .ml .mli .m4 .S .c .h \
		.sgml .tex .html .txt .bib \
		.dvi .ps .x .vo .zo .zi .za .i .s .o .a .b \
		.log .aux .bbl .toc .bst


############################ FIRST GOAL ###################################

ifndef CONFIGURATION
ifdef REQUIRE_CONFIG
ifeq ($(TUNES)/Makefile.config, $(wildcard $(TUNES)/Makefile.config))
include $(TUNES)/Makefile.config
else
CONFIGURATION = config
all: config
endif
endif
endif

ifndef CONFIGURATION
ifdef REQUIRE_DEPEND
ifeq (.depend,$(wildcard .depend))
else
CONFIGURATION = depend
all: depend
endif
endif
endif

ifndef CONFIGURATION
ifdef GOAL
all: $(GOAL)
endif
endif



############################ CONFIGURATION ###################################

CLEAN :=	*.o *.a *.s *.l *.x \
		*.lst *.crf *.xrf *.obj *.map *.exe \
		*~ '#*#' \
		core z?p foo bar quu*x test.pid)
VERYCLEAN := $(CLEAN) *.b
MRPROPER := $(VERYCLEAN) .depend

configuration: $(CONFIGURATION) $(TUNES)/Makefile.config

autoconf: $(TUNES)/bin/autoconf.sh
	$(ZSHELL) $(TUNES)/bin/autoconf.sh

config: autoconf

clean::
	$(RM) $(wildcard $(CLEAN))

veryclean::
	$(RM) $(wildcard $(VERYCLEAN))

mrproper::
	$(RM) $(wildcard $(MRPROPER))

dep: depend

depend:	Makefile.config $(TUNES)/bin/depend.pl
	$(PERL) $(TUNES)/bin/depend.pl > .depend


############################ Rules proper ###################################


%.html:	%.sgml
	sgml2html -l $*

%.txt:	%.sgml
	sgml2txt -l -f $*

%.aux %.toc:  %.tex
	$(LATEX) $*

%.bbl %.blg:  %.toc %.bib
	$(BIBTEX) $*

%.dvi:	%.tex # %.bbl
	$(LATEX) $*

%.ps:   %.dvi
	dvips -o $@ $<

%.x:	%.dvi phony
	xdvi $<

%.vo:	%.v
	coqc $<

%.s:	%.c
	$(CC) $(CFLAGS) -S -fverbose-asm -fasm -o $@ $<

%.o:	%.c
	$(CC) $(CFLAGS) -c -o $@ $<

#%.o:	%.s
#	$(AS) $(ASFLAGS) -o $@ $<

%.b:	%.o
	$(CC) $(LDFLAGS) -o $@ $<

############################### Version ################################

### Version number in file "version" ###
include $(TUNES)/version

# for linux-like versions, add a letter to the patch-level number instead
# of an internal version number :)
TUNESVER=$(MAJOR_VERSION).$(MINOR_VERSION).$(PATCH_LEVEL).$(INTERNAL_REVISION)


####################### Default values for commands #######################

ifndef SCHEME_TYPE
SCHEME_TYPE = stk
endif

ifndef SCHEME
SCHEME = snow
endif

ifndef SCHEME_FLAGS
SCHEME_FLAGS = -f
endif

ifndef CC
CC	= gcc			# Use "cc" if you don't have gcc
endif

ifndef CFLAGS
CFLAGS = -O2 -fasm -fomit-frame-pointer -Wall
endif

ifndef CPP
CPP	= gcc -E		# may be "cpp" (perhaps "/lib/cpp")
endif

ifndef LD
LD	= ld
endif

ifndef PERL
PERL = perl
endif

ifndef INSTALL_BIN
INSTALL_BIN="install -s -m 755"
endif

ifndef INSTALL_DIR
INSTALL_DIR="install -d -m 755"
endif

ifndef RM
RM = rm -f
endif

ifndef MV
MV	= mv
endif

ifndef CP
CP	= cp
endif

ifndef CHMOD
CHMOD	= chmod
endif

ifndef COMPRESS
COMPRESS = gzip -f9		# Go get gzip if you don't have it yet
COMPRSUFFIX = .gz
endif

ifndef ARCHIVETYPE
ARCHIVETYPE=tgz
endif

ifndef ZSHELL
ZSHELL = zsh -f
endif

ifndef SHELL
SHELL	= $(ZSHELL) -y
#SHELL	= /bin/sh
endif

ifndef ARCH
ARCH = `uname -m`
endif

ifndef CONFIG
CONFIG=linux-i386
endif

INSTTOOL = $(TUNES)/bin/$(ARCH)/tunesinsttool
HLLSCM_PATH = $(TUNES)/src/HLL/scm
SCHEME_LOADPATH = $(HLLSCM_PATH)
BUILD = $(ZSHELL) $(TUNES)/bin/build.sh
DEST_DIR = $(TUNES)/bin/$(ARCH)

export TUNES TUNESREL TUNESVER
export ZSHELL SHELL RM CP LN MV
export SCHEME_LOADPATH
export COMPRESS COMPRSUFFIX ARCHIVETYPE PATCH_PREFIX
