#!/bin/zsh -f

# If needed, see $TUNES/build.sh about zsh...

abort () {
  CODE=$1 ; shift
  print -r >&2 "$*"
  exit $CODE
}

### Verify that we are running from the right directory
#[[ -x ./autoconf.sh ]] ||
#  abort 1 "You should run autoconf.sh from the directory whence it lies"


### Messages ###
OKTEXT='
   Everything *looks* OK to me.
   If something isn'"'"'t really, you'"'"'ll have to manually configure Tunes.
'
HELPTEXT='
   If you manage getting Tunes to run on your system,
please send us (mailto:rideau@ens.fr) a description of how we could
autodetect and autoconfigure Tunes for it, so that next time you try,
automatic installation will succeed.
'

AUTODETECTFAILED='
   We could not autodetect what kind of system you have.
Define the OSTYPE and/or ARCH environment variables and
try autoconfiguring again.
'

UNKNOWNSYS='Unknown system. You must manually configure Tunes.
'

NOSCHEME='Could not find a supported Scheme interpreter.
'

MAKEFILE_CONFIG_HEADER='#
# This is your local Tunes/O'"'"'TOP Makefile configuration.
# If the autoconfiguration tool didn'"'"'t get something right,
# here is precisely where you should edit it.
# and of course, it would be nice if you could send us some
# information about how we could get it right for next release.
#'


### Initialization ###
#MODULES=()


### Function library ###
FIND_COMMAND () {
    COMMAND=$1
    name=$2
    shift 2
    if COMPATH=$(whence -p $name) ; then
	eval "$COMMAND='$COMPATH $*'"
    else
	print -r "Could not find command $2." >&2
    fi
}
FIND_SCHEME () {
  for scm in $SUPPORTED_SCHEMES ; do
    if SCHEME=$(whence -p $scm) ; then
      CONFIG_Scheme_$scm
      return 0
    fi
  done
  abort 4 "$NOSCHEME$HELPTEXT"
}

CONFIG_ARCH_i386 () {
        ARCH=i386
	CC=gcc
	CFLAGS="-O2 -fomit-frame-pointer -Wall"
}
CONFIG_OS_Linux () {
	FIND_COMMAND CP cp -f
	FIND_COMMAND ZSHELL zsh -f
#	FIND_COMMAND SHELL sh
	FIND_COMMAND RM rm -fv
	FIND_COMMAND INSTALL_DIR install -d -m 755
	FIND_COMMAND INSTALL_BIN install -s -m 755
}
CONFIG_Scheme_elk () {
	SCHEME_TYPE="elk"
	SCHEME_FLAGS="-l"
}
CONFIG_Scheme_stk () {
	SCHEME_TYPE="stk"
	SCHEME_FLAGS="-f"
}
CONFIG_Scheme_snow () { CONFIG_Scheme_stk }


DEFINE () {
  echo "# $1 $2"
  eval "VAL=\$$1"
  if [ -n "$VAL" ] ; then
    echo "I found that $1=$VAL" >&2
    echo "$1 = $VAL"
  fi
  echo
}

### Getting some system identifiers ###

if [ -n "$OSTYPE" ] ; then
  # The user gave us the type of the OS
  : some shells hate empty lists
elif [ -x /bin/uname ] ; then
  # BSD-style system identification
  OSTYPE="$(/bin/uname)"
else
  abort 2 "$AUTODETECTFAILED$HELPTEXT"
fi

if [ -n "$ARCH" ] ; then
  # The user gave us the type of the OS
  : some shells hate empty lists
elif [ -x /bin/arch ] ; then
  # BSD-style system identification
  ARCH="$(/bin/arch)"
else
  abort 2 "$AUTODETECTFAILED$HELPTEXT"
fi


### Parsing the identification string for useful information ###

case "$OSTYPE:$ARCH" in
  [Ll]inux*:i?86)
	CONFIG=linux-i386
	CONFIG_OS_Linux
	CONFIG_ARCH_i386
	#CONFIG_ncurses		# should find & configure ncurses
	#CONFIG_X11		# should find & configure X11
	#CONFIG_SVGALIB		# should find & configure SVGALIB
	;;
#  sunos4:sun4)
#	CONFIG_OS_sunos4
#	CONFIG_ARCH_sun4
#	;;
  *)
	abort 2 "$UNKNOWNSYS$HELPTEXT"
	;;
esac


### Looking for a supported Scheme interpreter ###

SUPPORTED_SCHEMES=(snow stk) # elk

FIND_SCHEME

{
echo "$MAKEFILE_CONFIG_HEADER"
DEFINE OSTYPE	"defines the type of the underlying operating system"
DEFINE ARCH	"defines the type of the hardware architecture (CPU, etc)"
DEFINE CONFIG	"is the name of the main configuration-dependent file"
DEFINE ZSHELL	"is the path for the Z shell"
DEFINE SCHEME_TYPE	"is the type of scheme interpreter"
DEFINE SCHEME	"is the path for a scheme interpreter"
DEFINE SCHEME_FLAGS	"is command line flags for $SCHEME"
DEFINE CC	"is the path to access the C compiler"
DEFINE CFLAGS	"is the flags for the C compiler"
DEFINE CP	"is a command to copy files"
DEFINE RM	"is a command to delete files"
DEFINE INSTALL_DIR	"is a command to install directories"
DEFINE INSTALL_BIN	"is a command to install executable files"
#DEFINE MODULES		"is the list of additional modules to build into Tunes
} > ${TUNES}/Makefile.config

echo "$OKTEXT$HELPTEXT"
