#!/usr/local/bin/perl -w

# Computes Tunes dependencies.
#
# Currently supported:
# * provide and require or features in .scm files,
#  with at most one provided feature per file.
#  creates Makefile entries like
#	.PHONY:		feature
#	feature:	feature.scm requiredfeature1 requiredfeature2

use strict;


my $SCHEME_LOADPATH = $ENV{"SCHEME_LOADPATH"} ;

my @SCM_LOADPATH = (".",(split /:/, $SCHEME_LOADPATH)) ;
my @HLM_LOADPATH = (".","config","langC") ;
my @FEATURE_LIST = ();
my %DEPENDENCIES = ();

my %SCM_DONE = () ;
my %HLM_DONE = () ;
my @SCM_QUEUE = ("tunes.config") ;
my @HLM_QUEUE = () ;

my $DEBUG = 1;
$| = $DEBUG;



sub errprint {
    print STDERR join(" ",@_)."\n";
}
sub dbg {
    if($DEBUG) {
	errprint @_;
    }
}
dbg ((join "\n",@SCM_LOADPATH), "\n\n");
dbg ((join "\n",@HLM_LOADPATH), "\n\n");

sub lookup_path ($@) {
    my $file = shift ;
    my $path ;
    if ( -f $file ) { return $file }
    while ( $path = shift ) {
	if ( -f "$path/$file" ) { return "$path/$file" }
    }
    return
}
sub lookup_scm_path ($) { lookup_path shift,@SCM_LOADPATH ; }
sub lookup_hlm_path ($) { lookup_path shift,@HLM_LOADPATH ; }

sub do_queue ($\@\%) {
    my ($elem,$queue,$done) = @_ ;
    if (not defined $$done{$elem}) {
	push @$queue,$elem;
    }
}
sub do_unqueue (\@\%) {
    my ($queue,$done) = @_ ;
    my $elem = shift @$queue ;
    $$done{$elem} = 1 ;
    return $elem ;
}
sub queue_scm ($) { do_queue shift,@SCM_QUEUE,%SCM_DONE ; }
sub queue_hlm ($) { do_queue shift,@HLM_QUEUE,%HLM_DONE ; }
sub unqueue_scm ($) { do_unqueue @SCM_QUEUE,%SCM_DONE ; }
sub unqueue_hlm ($) { do_unqueue @HLM_QUEUE,%HLM_DONE ; }

sub norm_atom ($) {
    local $_ = shift ;
    s/^\'// || s/^\"(.*)\"/$1/ || s/^.*$// ;
    $_
}
sub norm_scm ($) {
    local $_ = norm_atom shift ;
    if ( /./ && ! /\.scm/) { $_ .= ".scm" }
    $_
}
sub norm_hlm ($) {
    local $_ = norm_atom shift ;
    if ( /./ && ! /\.hlm/) { $_ .= ".hlm" }
    $_
}
sub depend_scm_file {
    my $file = shift;
    dbg "dsf: $file";
    open FILE, "<$file";
    my @required = ();
    while(<FILE>) {
	if(/\((load|require) *([^ ()]*)\)/i) {
	    my $scm = norm_scm $2 ;
	    if ( $scm ne "" ) {
		#dbg "detected required feature $scm in $file...";
		my $dep = lookup_scm_path $scm ;
		if ( $dep ne "" ) {
		    queue_scm $dep;
		    push @required, $dep;
		} else {
		    errprint "Could not find file $scm loaded by $file\n";
		}
	    }
	}
	if( /\(hlm\-(load|require) *([^ ()]*)\)/i ) {
	    my $hlm = norm_hlm $2 ;
	    if ( $hlm ne "" ) {
		#dbg "detected required feature $hlm in $file...";
		my $dep = lookup_hlm_path $hlm ;
		if ( $dep ne "" ) {
		    queue_hlm $dep;
		    push @required, $dep;
		} else {
		    errprint "Could not find file $hlm required by $file\n";
		}
	    }
	}
    }
    dbg "dsfr: ",@required;
    $DEPENDENCIES{$file} = \@required ;
    close FILE;
}
sub depend_hlm_file {
    my $file = shift;
    dbg "dhf: $file";
    open FILE, "<$file";
    my @required = ();
    while(<FILE>) {
	if(/\((load|require) *([^ ()]*)\)/i) {
	    my $hlm = norm_hlm $2 ;
	    if ( $hlm ne "" ) {
		#dbg "detected required feature $hlm in $file...";
		my $dep = lookup_hlm_path $hlm ;
		if ( $dep ) {
		    queue_hlm $dep;
		    push @required, $dep;
		} else {
		    errprint "Could not find file $hlm required by $file\n";
		}
	    }
	}
    }
    dbg "dhfr: ",@required;
    $DEPENDENCIES{$file} = \@required ;
    close FILE;
}

while ( $#SCM_QUEUE + $#HLM_QUEUE > -2 ) {
    if ( $#SCM_QUEUE > -1 ) {
	depend_scm_file &unqueue_scm ;
    }
    if ( $#HLM_QUEUE > -1 ) {
	depend_hlm_file &unqueue_hlm ;
    }
}

my ($file,$deps);

while (($file,$deps) = each %DEPENDENCIES) {
    print ".PHONY: $file.foo\n"
}
while (($file,$deps) = each %DEPENDENCIES) {
    print "$file.foo: $file ", (join " ",map {"$_.foo"} @$deps), "\n";
}
